;  (C) 2006 - Michel Manceron <michel.manceron@siemens.com>) 
;           - Vincent Besanon <elguapito@free.fr>
; Quick & (not so) Dirty MSI package for ZABBIX 1.1.4 client for Windows

;--------------------------------
;Includes

  ; Modern UI
  !include "MUI.nsh"
  ; Text Functions
  !include "TextFunc.nsh"
  ; LogicLib
  !include "LogicLib.nsh"
  ; StrCase (see Include\StrCase.nsh for author info)
  !include ".\Include\StrCase.nsh"
  ;
  !include ".\include\GetParameters.nsh"
  !include ".\include\strstr.nsh"
  !include ".\include\GetParameterValue.nsh"
;--------------------------------
;General
  !define NAME  "Zabbix Agent"
  !define VERSION "1.1.4"
  !define VER_PACK "1.1.4.0"
  ;Name and file
  Name "${NAME}"
  OutFile "zabbix_agent-${VERSION}_installer.exe"
  
  ;Default installation folder
  InstallDir "$PROGRAMFILES\${NAME}"
  
  ; Product Information
  VIAddVersionKey "ProductName" "${NAME}"
  VIAddVersionKey "CompanyName" "ZABBIX"
  VIAddVersionKey "LegalCopyright" " ZABBIX SIA"
  VIAddVersionKey "FileDescription" "${NAME} Installer 32 and 64 bits"
  VIAddVersionKey "FileVersion" "${VERSION}"

  VIProductVersion "${VER_PACK}"

; Registry key to check for directory (so if you install again, it will 
  ; overwrite the old one automatically)
  InstallDirRegKey HKLM "Software\${NAME}" "Install_Dir"

;--------------------------------
;Variables definitions

Var OUT
Var HOST
Var SERVER
Var RMTCMD
Var TMPHOST
Var OS64B
;--------------------------------
;Macros definitions
!macro PARAMETERNAME ParamName DefaultValue
  Push "${ParamName}" 
  Push "${DefaultValue}" 
  Call GetParameterValue
  Pop $${ParamName}
!macroend


!insertmacro ConfigWrite

;--------------------------------
;Interface Settings
  !define MUI_ICON ".\img\zabbix.ico"
  !define MUI_UNICON ".\img\zabbix.ico"
  !define MUI_HEADERIMAGE
  !define MUI_HEADERIMAGE_BITMAP ".\img\zabbix_logo.bmp"
  !define MUI_ABORTWARNING

;--------------------------------
;Pages

  !insertmacro MUI_PAGE_WELCOME
  !insertmacro MUI_PAGE_LICENSE ".\Installer\docs\LICENSE.txt"
  !insertmacro MUI_PAGE_COMPONENTS
  ; Custom page to set some options in config file
  Page custom ConfigOptions
  !insertmacro MUI_PAGE_DIRECTORY
  !insertmacro MUI_PAGE_INSTFILES
  
  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES

;--------------------------------
;Languages
 
  !insertmacro MUI_LANGUAGE "English"

;--------------------------------
;Reserve Files
  
  ;These files should be inserted before other files in the data block
  ;Keep these lines before any File command
  ;Only for solid compression (by default, solid compression is enabled for BZIP2 and LZMA)
  
  ReserveFile "ihm.ini"
  !insertmacro MUI_RESERVEFILE_INSTALLOPTIONS

;--------------------------------
;Installer Sections

Section "Zabbix Agent (required)" SEC01
  
  SectionIn RO
  
  ; Zabbix binaries & config file
  SetOutPath "$INSTDIR"
  strcmp $OS64B '0' 0 x64Bitinst
    File ".\Installer\ZabbixW32.exe"
    goto s1
  x64Bitinst:
    File ".\Installer\ZabbixW64.exe"
  s1:
  File ".\Installer\zabbix_agentd.conf"
  ; Zabbix docs & license
  File ".\Installer\docs\*"
  ; Read user input options
  
  !insertmacro MUI_INSTALLOPTIONS_READ $SERVER "ihm.ini" "Field 2" "State"
  !insertmacro MUI_INSTALLOPTIONS_READ $HOST   "ihm.ini" "Field 6" "State"
  !insertmacro MUI_INSTALLOPTIONS_READ $RMTCMD "ihm.ini" "Field 7" "State"
  
    
  ; Puts some info based on host in config file
  ${ConfigWrite} '$INSTDIR\zabbix_agentd.conf' 'Server=' '$SERVER' $OUT
  ${ConfigWrite} '$INSTDIR\zabbix_agentd.conf' 'Hostname=' '$HOST' $OUT
  ${ConfigWrite} '$INSTDIR\zabbix_agentd.conf' 'EnableRemoteCommands=' '$RMTCMD' $OUT
  ${ConfigWrite} '$INSTDIR\zabbix_agentd.conf' '#LogFile=' '$INSTDIR/Zabbix_agentd.log' $OUT
  ${ConfigWrite} '$INSTDIR\zabbix_agentd.conf' '#PidFile=' '$INSTDIR/Zabbix_agentd.pid' $OUT

  strcmp $OS64B '0' 0 x64BitSrv
    DetailPrint "Registering Zabbix Agent in services..."
    nsExec::ExecToStack '"$INSTDIR\ZabbixW32.exe" --config "$INSTDIR\zabbix_agentd.conf" install'
    DetailPrint "Starting Zabbix Agent..."
    nsExec::ExecToStack '"$INSTDIR\ZabbixW32.exe" start'
    goto S2
  x64BitSrv:
    DetailPrint "Registering Zabbix64 Agent in services..."
    nsExec::ExecToStack '"$INSTDIR\ZabbixW64.exe" --config "$INSTDIR\zabbix_agentd.conf" install'
    DetailPrint "Starting Zabbix64 Agent..."
    nsExec::ExecToStack '"$INSTDIR\ZabbixW64.exe" start'
  S2:
  ; Write the installation path into the registry
  WriteRegStr HKLM "SOFTWARE\${NAME}" "Install_Dir" "$INSTDIR"
  
  ; Write the uninstall keys for Windows
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME}" "DisplayName" "${NAME} ${VERSION}"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME}" "UninstallString" '"$INSTDIR\uninstall.exe"'
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME}" "NoModify" 1
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME}" "NoRepair" 1
  WriteUninstaller "uninstall.exe"
SectionEnd

Section "Zabbix Sender (optional)" SEC02
  SetOutPath "$INSTDIR"
  
  ; ADD FILES
  DetailPrint "Installing optional Zabbix Sender..."
  File ".\Installer\zabbix_sender.exe"
SectionEnd

;--------------------------------
; Descriptions

LangString DESC_SEC01 ${LANG_ENGLISH} "Install ${NAME}windows (32 or 64 bits) platform"
LangString DESC_SEC02 ${LANG_ENGLISH} "Install Zabbix Sender to send custom value via command line"

!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
  !insertmacro MUI_DESCRIPTION_TEXT ${SEC01} $(DESC_SEC01)
  !insertmacro MUI_DESCRIPTION_TEXT ${SEC02} $(DESC_SEC02)
!insertmacro MUI_FUNCTION_DESCRIPTION_END

;--------------------------------
;InstallerOptions (custom IHM) Functions

Function .onInit
  ;Extract InstallOptions INI files
  System::Call "kernel32::GetCurrentProcess() i .s"
  System::Call "kernel32::IsWow64Process(i s, *i .r0)"
  strcpy $OS64B $0
  !insertmacro MUI_INSTALLOPTIONS_EXTRACT "ihm.ini"
  !insertmacro PARAMETERNAME SERVER NONE
  ifsilent 0 OninitOK
   strcmp $SERVER 'NONE' 0 OninitOK
   Abort
   OninitOK:
  !insertmacro PARAMETERNAME RMTCMD 0
   StrCmp $RMTCMD 1 onInitEtap1 0
   strcpy $RMTCMD 0
  onInitEtap1:
   ReadEnvStr $TMPHOST COMPUTERNAME
   ${StrCase} $HOST $TMPHOST "L"
  !insertmacro MUI_INSTALLOPTIONS_WRITE "ihm.ini" "Field 2" "State" $SERVER
  !insertmacro MUI_INSTALLOPTIONS_WRITE "ihm.ini" "Field 6" "State" $HOST
  !insertmacro MUI_INSTALLOPTIONS_WRITE "ihm.ini" "Field 7" "State" $RMTCMD
  ; Uninstall if exist
   ReadRegStr $R0 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME}" "UninstallString"
   StrCmp $R0 "" onIniTCont
   ifsilent onIniUnsint
     MessageBox MB_OKCANCEL|MB_ICONEXCLAMATION "${NAME} is already installed. $\n$\nClick `OK` to remove the previous version or `Cancel` to cancel this upgrade." IDOK onIniUnsint
     Abort   
  onIniUnsint:
    ExecWait '$R0 /S _?=$INSTDIR'
  onIniTCont:

FunctionEnd

LangString TEXT_IO_TITLE ${LANG_ENGLISH} "Configuration"
LangString TEXT_IO_SUBTITLE ${LANG_ENGLISH} "Set some options in the config file."

Function ConfigOptions
  ; Read value from %COMPUTERNAME% environment variable and store it in $HOST variable
  !insertmacro PARAMETERNAME SERVER NONE
  !insertmacro PARAMETERNAME RMTCMD 0
   StrCmp $RMTCMD 1 ConfigOptionsEtap1 0
   strcpy $RMTCMD 0
   ConfigOptionsEtap1:
   ReadEnvStr $TMPHOST COMPUTERNAME
   ${StrCase} $HOST $TMPHOST "L"
  !insertmacro MUI_HEADER_TEXT "$(TEXT_IO_TITLE)" "$(TEXT_IO_SUBTITLE)"
  ; Show default or detected value
  
  !insertmacro MUI_INSTALLOPTIONS_WRITE "ihm.ini" "Field 6" "State" $HOST
  !insertmacro MUI_INSTALLOPTIONS_WRITE "ihm.ini" "Field 2" "State" $SERVER 
  !insertmacro MUI_INSTALLOPTIONS_WRITE "ihm.ini" "Field 7" "State" $RMTCMD
  ; Show custom page
  !insertmacro MUI_INSTALLOPTIONS_DISPLAY "ihm.ini"
FunctionEnd

;--------------------------------
; Uninstaller

UninstallText "This will uninstall ${NAME}. Hit next to continue."

Section "Uninstall"
  
  ; Remove registry keys
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${NAME}"
  DeleteRegKey HKLM "SOFTWARE\${NAME}"
  System::Call "kernel32::GetCurrentProcess() i .s"
  System::Call "kernel32::IsWow64Process(i s, *i .r0)"
  strcmp $0 '0' 0 x64stop 
  
  ; Stop and remove Zabbix Agent service
    DetailPrint "Stopping Zabbix Agent service..."
    nsExec::ExecToStack '"$INSTDIR\ZabbixW32.exe" stop'
    DetailPrint "Removing Zabbix Agent..."
    nsExec::ExecToStack '"$INSTDIR\ZabbixW32.exe" remove'
    Goto S_Stop
    
  x64stop:
    DetailPrint "Stopping Zabbix64 Agent service..."
    nsExec::ExecToStack '"$INSTDIR\ZabbixW64.exe" stop'
    DetailPrint "Removing Zabbix64 Agent..."
    nsExec::ExecToStack '"$INSTDIR\ZabbixW64.exe" remove'
  S_Stop:
  ; Remove files and uninstaller
  Sleep 2000
  Delete "$INSTDIR\*"

  ; Remove install directory
  RMDir "$INSTDIR"

SectionEnd

